@@ -19,8 +19,10 @@ from tqdm.auto import tqdm
 from collections import defaultdict
 import warnings
 from functools import partial
+import gc
 
-from .utils import MODEL_CLASSES, is_module_installed, InvalidBenchmark
+from .utils import (MODEL_CLASSES, is_module_installed, InvalidBenchmark,
+                    TwolabelTrainer)
 
 
 class BaseBenchmark(ABC):
@@ -79,6 +81,8 @@ class BaseBenchmark(ABC):
                  epochs: int = 5,
                  warmup_steps: int = 50,
                  batch_size: int = 32,
+                 multilabel: bool = False,
+                 split_point: Optional[int] = None,
                  verbose: bool = False):
         if batch_size not in [1, 2, 4, 8, 16, 32]:
             raise TypeError('The batch size must be either 1, 2, 4, 8, '
@@ -92,6 +96,8 @@ class BaseBenchmark(ABC):
         self.warmup_steps = warmup_steps
         self.num_labels = num_labels
         self.id2label = id2label
+        self.multilabel = multilabel
+        self.split_point = split_point
         self.verbose = verbose
         if self.id2label is not None:
             self.label2id = {label: id for id, label in enumerate(id2label)}
@@ -106,6 +112,7 @@ class BaseBenchmark(ABC):
     @staticmethod
     def _get_stats(metrics: Dict[str, List[Dict[str, float]]],
                    metric_name: str,
+                   num_samples: int,
                    split: str) -> Tuple[float, float]:
         '''Helper function to compute the mean with confidence intervals.
 
@@ -117,6 +124,8 @@ class BaseBenchmark(ABC):
             metric_name (str):
                 The name of the metric. Is used to collect the correct metric
                 from `metrics`.
+            num_samples (int):
+                The number of samples in the data the model is evaluated on.
             split (str):
                 The dataset split we are calculating statistics of. Is used to
                 collect the correct metric from `metrics`.
@@ -128,12 +137,21 @@ class BaseBenchmark(ABC):
         '''
         key = f'{split}_{metric_name}'
         metric_values = [dct[key] for dct in metrics[split]]
+
+        # Set Z-value to 1.96, corresponding to a 95% confidence interval
+        z = 1.96
+
         mean = np.mean(metric_values)
+        std_err = np.sqrt(mean * (1 - mean) / num_samples)
+
         with warnings.catch_warnings():
             warnings.simplefilter('ignore')
-            sample_std = np.std(metric_values, ddof=1)
-            std_err = sample_std / np.sqrt(len(metric_values))
-            return mean, 1.96 * std_err
+
+            if len(metric_values) > 1:
+                sample_std = np.std(metric_values, ddof=1)
+                std_err += sample_std / np.sqrt(len(metric_values))
+
+        return mean, z * std_err
 
     def _load_model(self,
                     model_id: str,
@@ -211,7 +229,7 @@ class BaseBenchmark(ABC):
                 model = model_cls.from_pretrained(model_id,
                                                   config=config,
                                                   cache_dir=self.cache_dir)
-            except OSError:
+            except (OSError, ValueError):
                 raise InvalidBenchmark(f'The model {model_id} could not be '
                                        f'loaded from the HuggingFace hub.')
 
@@ -309,6 +327,8 @@ class BaseBenchmark(ABC):
     @abstractmethod
     def _log_metrics(self,
                      metrics: Dict[str, List[Dict[str, float]]],
+                     num_train: int,
+                     num_test: int,
                      model_id: str):
         '''Log the metrics.
 
@@ -317,6 +337,10 @@ class BaseBenchmark(ABC):
                 The metrics that are to be logged. This is a dict with keys
                 'train' and 'test', with values being lists of dictionaries
                 full of metrics.
+            num_train (int):
+                The number of training samples.
+            num_test (int):
+                The number of test samples.
             model_id (str):
                 The full HuggingFace Hub path to the pretrained transformer
                 model.
@@ -480,27 +504,36 @@ class BaseBenchmark(ABC):
 
             metrics = defaultdict(list)
             for _ in itr:
-
-                # Reinitialise a new model
-                model = self._load_model(model_id, **model_metadata)['model']
-
-                # Initialise Trainer
-                compute_metrics = partial(self._compute_metrics,
-                                          id2label=model.config.id2label)
-                trainer = Trainer(model=model,
-                                  args=training_args,
-                                  train_dataset=preprocessed_train,
-                                  tokenizer=tokenizer,
-                                  data_collator=data_collator,
-                                  compute_metrics=compute_metrics)
-
-                # Remove the callback which prints the metrics after each
-                # evaluation
-                trainer.remove_callback(PrinterCallback)
-
-                # Finetune the model
                 while True:
                     try:
+                        # Reinitialise a new model
+                        model = self._load_model(model_id,
+                                                 **model_metadata)['model']
+
+                        # Initialise compute_metrics function
+                        compute_metrics = partial(
+                            self._compute_metrics,
+                            id2label=model.config.id2label
+                        )
+
+                        # Initialise Trainer
+                        trainer_args = dict(model=model,
+                                            args=training_args,
+                                            train_dataset=preprocessed_train,
+                                            tokenizer=tokenizer,
+                                            data_collator=data_collator,
+                                            compute_metrics=compute_metrics)
+                        if self.multilabel:
+                            trainer_args['split_point'] = self.split_point
+                            trainer = TwolabelTrainer(**trainer_args)
+                        else:
+                            trainer = Trainer(**trainer_args)
+
+                        # Remove the callback which prints the metrics after
+                        # each evaluation
+                        trainer.remove_callback(PrinterCallback)
+
+                        # Finetune the model
                         if task == 'fill-mask':
                             trainer.train()
 
@@ -524,8 +557,8 @@ class BaseBenchmark(ABC):
                         bs = training_args.per_device_train_batch_size
                         ga = training_args.gradient_accumulation_steps
                         if bs == 1:
-                            raise RuntimeError('CUDA out of memory, even '
-                                               'with a batch size of 1!')
+                            raise InvalidBenchmark('CUDA out of memory, even '
+                                                   'with a batch size of 1!')
                         training_args.per_device_train_batch_size = bs // 2
                         training_args.gradient_accumulation_steps = ga * 2
                         trainer.args = training_args
@@ -533,7 +566,16 @@ class BaseBenchmark(ABC):
                 metrics['train'].append(train_metrics)
                 metrics['test'].append(test_metrics)
 
-            self._log_metrics(metrics, model_id=model_id)
+            self._log_metrics(metrics,
+                              model_id=model_id,
+                              num_train=len(train),
+                              num_test=len(test))
+
+            # Garbage collection, to avoid memory issues
+            del model, model_dict
+            del preprocessed_train, preprocessed_test, train, test
+            gc.collect()
+
             return metrics
 
         elif framework == 'spacy':
@@ -565,7 +607,16 @@ class BaseBenchmark(ABC):
                             for key, val in test_metrics.items()}
             metrics = dict(train=[train_metrics], test=[test_metrics])
 
-            self._log_metrics(metrics, model_id=model_id)
+            self._log_metrics(metrics,
+                              model_id=model_id,
+                              num_train=len(train),
+                              num_test=len(test))
+
+            # Garbage collection, to avoid memory issues
+            del model, model_dict
+            del preprocessed_train, preprocessed_test, train, test
+            gc.collect()
+
             return metrics
 
         else:
