@@ -28,23 +28,34 @@ class Benchmark:
     '''Benchmarking all the Scandinavian language models.
 
     Args
-        language (str or list of str, optional):
-            The language codes of the languages to include in the list.
-            Defaults to ['da', 'sv', 'no'].
-        task (str or list of str, optional):
-            The tasks to consider in the list. Defaults to ['fill-mask',
-            'token-classification', 'text-classification'].
+        language (str or list of str or None, optional):
+            The language codes of the languages to include in the list. If None
+            then all languages will be considered.  Defaults to ['da', 'sv',
+            'no', 'is', 'fo'].
+        task (str or list of str or None, optional):
+            The tasks to consider in the list. If None then all tasks will be
+            considered. Defaults to None.
     '''
     def __init__(self,
-                 language: Union[str, List[str]] = ['da', 'sv', 'no'],
-                 task: Union[str, List[str]] = ['fill-mask',
-                                                'token-classification',
-                                                'text-classification'],
+                 language: Optional[Union[str, List[str]]] = ['da', 'sv', 'no',
+                                                              'is', 'fo'],
+                 task: Optional[Union[str, List[str]]] = None,
                  batch_size: int = 32,
                  verbose: bool = False):
-        self.languages = [language] if isinstance(language, str) else language
-        self.tasks = [task] if isinstance(task, str) else task
-        self._model_lists = self._get_model_lists()
+
+        # Ensure that `self.languages` is a list
+        if isinstance(language, str) or language is None:
+            self.languages = [language]
+        else:
+            self.languages = language
+
+        # Ensure that `self.tasks` is a list
+        if isinstance(task, str) or task is None:
+            self.tasks = [task]
+        else:
+            self.tasks = task
+
+        self._model_lists = None
         self.benchmark_results = defaultdict(dict)
         params = dict(verbose=verbose, batch_size=batch_size)
         self._benchmarks = [
@@ -70,18 +81,27 @@ class Benchmark:
         logger.setLevel(logging_level)
 
     @staticmethod
-    def _get_model_ids(language: str, task: str) -> List[str]:
+    def _get_model_ids(language: Optional[str] = None,
+                       task: Optional[str] = None) -> List[str]:
         '''Retrieves all the model IDs in a given language with a given task.
 
         Args:
-            language (str): The language code of the language to consider.
-            task (str): The task to consider.
+            language (str or None):
+                The language code of the language to consider. If None then the
+                models will not be filtered on language. Defaults to None.
+            task (str or None):
+                The task to consider. If None then the models will not be
+                filtered on task. Defaults to None.
 
         Returns:
             list of str: The model IDs of the relevant models.
         '''
         url = 'https://huggingface.co/models'
-        params = dict(filter=language, pipeline_tag=task)
+        params = dict()
+        if language is not None:
+            params['filter'] = language
+        if task is not None:
+            params['pipeline_tag'] = task
         html = requests.get(url, params=params).text
         soup = BeautifulSoup(html, 'html.parser')
         articles = soup.find_all('article')
@@ -96,6 +116,18 @@ class Benchmark:
     def _get_model_lists(self) -> Dict[str, List[str]]:
         '''Updates the model list'''
 
+        # Log fetching message
+        log_msg = 'Fetching list of models'
+        if None not in self.languages:
+            log_msg += f' for the languages {self.languages}'
+            if None not in self.tasks:
+                log_msg += f' and tasks {self.tasks}'
+        else:
+            if None not in self.tasks:
+                log_msg += f' for the tasks {self.tasks}'
+        log_msg += ' from the HuggingFace Hub.'
+        logger.info(log_msg)
+
         # Initialise model lists
         model_lists = defaultdict(list)
         for language in self.languages:
@@ -141,6 +173,8 @@ class Benchmark:
                 having the model IDs as keys.
         '''
         if model_id is None:
+            if self._model_lists is None:
+                self._model_lists = self._get_model_lists()
             model_ids = self._model_lists['all']
         elif isinstance(model_id, str):
             model_ids = [model_id]
