@@ -4,7 +4,6 @@ from datasets import Dataset
 import numpy as np
 from typing import Tuple, Dict, List, Optional
 import logging
-import itertools as it
 
 from .token_classification import TokenClassificationBenchmark
 from .datasets import load_ddt_dep
@@ -114,30 +113,35 @@ class DdtDepBenchmark(TokenClassificationBenchmark):
                         'root',
                         'vocative',
                         'xcomp']
-        id2label = list(map(str, it.product(id2label_head, id2label_dep)))
+        id2label = id2label_head + id2label_dep
         super().__init__(epochs=5,
                          id2label=id2label,
                          cache_dir=cache_dir,
                          learning_rate=learning_rate,
                          warmup_steps=warmup_steps,
                          batch_size=batch_size,
+                         multilabel=True,
+                         split_point=len(id2label_head),
                          verbose=verbose)
 
     @doc_inherit
     def _load_data(self) -> Tuple[Dataset, Dataset]:
         X_train, X_test, y_train, y_test = load_ddt_dep()
-        train_labels = [list(map(str, zip(heads, deps)))
-                        for heads, deps in zip(y_train['heads'],
-                                               y_train['deps'])]
-        test_labels = [list(map(str, zip(heads, deps)))
-                       for heads, deps in zip(y_test['heads'],
-                                              y_test['deps'])]
+
+        train_labels = [list(zip(head, dep))
+                        for head, dep in zip(y_train['heads'],
+                                             y_train['deps'])]
+        test_labels = [list(zip(head, dep))
+                       for head, dep in zip(y_test['heads'],
+                                            y_test['deps'])]
+
         train_dict = dict(doc=X_train['doc'],
                           tokens=X_train['tokens'],
                           orig_labels=train_labels)
         test_dict = dict(doc=X_test['doc'],
                          tokens=X_test['tokens'],
                          orig_labels=test_labels)
+
         train = Dataset.from_dict(train_dict)
         test = Dataset.from_dict(test_dict)
         return train, test
@@ -149,57 +153,116 @@ class DdtDepBenchmark(TokenClassificationBenchmark):
         # Get the predictions from the model
         predictions, labels = predictions_and_labels
 
+        # If `id2label` is given then assume that `predictions` contain ID
+        # logits for every token, where an ID can mean both a head and a dep,
+        # so it needs to be split up in these two halves.
         if id2label is not None:
-            raw_predictions = np.argmax(predictions, axis=-1)
 
-            # Remove ignored index (special tokens)
-            predictions = [
+            # Here we split up the predictions into the "head part" and the
+            # "dep part"
+            predictions1 = predictions[:, :, :self.split_point]
+            predictions2 = predictions[:, :, self.split_point:]
+
+            # With the predictions split up, we can then get the highest logits
+            # to get the head and dep label
+            raw_predictions1 = np.argmax(predictions1, axis=-1)
+            raw_predictions2 = np.argmax(predictions2, axis=-1)
+
+            # The `labels` are assumed to be of shape
+            # (batch_size, sequence_length, label_type), where `label_type` is
+            # a binary number indicating either the head label or the dep
+            # label. Here we extract the two different labels.
+            labels1 = labels[:, :, 0]
+            labels2 = labels[:, :, 1]
+
+            # Remove ignored indices from predictions and labels
+            predictions1 = [
                 [id2label[pred] for pred, lbl in zip(prediction, label)
                  if lbl != -100]
-                for prediction, label in zip(raw_predictions, labels)
+                for prediction, label in zip(raw_predictions1, labels1)
             ]
-            labels = [
-                [id2label[lbl] for _, lbl in zip(prediction, label)
+            predictions2 = [
+                [id2label[pred] for pred, lbl in zip(prediction, label)
                  if lbl != -100]
-                for prediction, label in zip(raw_predictions, labels)
+                for prediction, label in zip(raw_predictions2, labels2)
             ]
+            labels1 = [[id2label[lbl] for lbl in label if lbl != -100]
+                       for label in labels1]
+            labels2 = [[id2label[lbl] for lbl in label if lbl != -100]
+                       for label in labels2]
 
-        results = self._metric.compute(predictions=predictions,
-                                       references=labels)
-        return dict(accuracy=results['overall_accuracy'])
+            # Next merge the predictions and labels, so that we have a pair of
+            # predicted/gold labels for each token
+            predictions_merged = [list(map(str, zip(head, dep)))
+                                  for head, dep in zip(predictions1,
+                                                       predictions2)]
+            labels_merged = [list(map(str, zip(head, dep)))
+                             for head, dep in zip(labels1, labels2)]
+
+        # If `id2label` is not given then assume that the predictions and
+        # labels contain a pair (head, dep) for every token.
+        else:
+            # Convert the pair of labels to a single one by converting it into
+            # strings. This is used in LAS computations.
+            predictions_merged = [list(map(str, tuples))
+                                  for tuples in predictions]
+            labels_merged = [list(map(str, tuples)) for tuples in labels]
+
+            # Extract the heads predictions and labels, used in UAS computation
+            predictions1 = [[head for head, _ in preds]
+                            for preds in predictions]
+            labels1 = [[head for head, _ in label_list]
+                       for label_list in labels]
+
+        # Compute metrics for the heads, which is used in UAS computation
+        results_head = self._metric.compute(predictions=predictions1,
+                                            references=labels1)
+
+        # Compute metrics for the merged heads and deps, which is used in LAS
+        # computation
+        results_merged = self._metric.compute(predictions=predictions_merged,
+                                              references=labels_merged)
+
+        # Extract UAS and LAS and return them
+        uas = results_head['overall_accuracy']
+        las = results_merged['overall_accuracy']
+        return dict(uas=uas, las=las)
 
     @doc_inherit
     def _log_metrics(self,
                      metrics: Dict[str, List[Dict[str, float]]],
+                     num_train: int,
+                     num_test: int,
                      model_id: str):
-        kwargs = dict(metrics=metrics, metric_name='accuracy')
-        train_mean, train_std_err = self._get_stats(split='train', **kwargs)
-        test_mean, test_std_err = self._get_stats(split='test', **kwargs)
 
-        # Multiply scores by x100 to make them easier to read
-        train_mean *= 100
-        test_mean *= 100
-        train_std_err *= 100
-        test_std_err *= 100
+        for metric_name in ['uas', 'las']:
+            kwargs = dict(metrics=metrics, metric_name=metric_name)
+            train_mean, train_se = self._get_stats(split='train',
+                                                   num_samples=num_train,
+                                                   **kwargs)
+            test_mean, test_se = self._get_stats(split='test',
+                                                 num_samples=num_test,
+                                                 **kwargs)
 
-        if not np.isnan(train_std_err):
-            msg = (f'Mean LAS on the DEP part of DDT for {model_id}:\n'
-                   f'  - Train: {train_mean:.2f} ± {train_std_err:.2f}\n'
-                   f'  - Test: {test_mean:.2f} ± {test_std_err:.2f}')
-        else:
-            msg = (f'LAS on the DEP part of DDT for {model_id}:\n'
-                   f'  - Train: {train_mean:.2f}\n'
-                   f'  - Test: {test_mean:.2f}')
+            # Multiply scores by x100 to make them easier to read
+            train_mean *= 100
+            test_mean *= 100
+            train_se *= 100
+            test_se *= 100
 
-        logger.info(msg)
+            msg = (f'{metric_name.upper()} on the DEP part of DDT '
+                   f'for {model_id}:\n'
+                   f'  - Train: {train_mean:.2f} ± {train_se:.2f}\n'
+                   f'  - Test: {test_mean:.2f} ± {test_se:.2f}')
+            logger.info(msg)
 
     @doc_inherit
     def _get_spacy_token_labels(self, processed) -> List[str]:
-        def get_heads_and_deps(token) -> Tuple[str, str]:
+        def get_heads_and_deps(token) -> List[str]:
             dep = token.dep_.lower().replace('obl:loc', 'obl:lmod')
             if dep == 'root':
                 head = '0'
             else:
                 head = str(token.head.i + 1)
-            return (head, dep)
-        return [str(get_heads_and_deps(token)) for token in processed]
+            return [head, dep]
+        return [get_heads_and_deps(token) for token in processed]
